package com.fastaccess.ui.adapter.viewholder;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import android.view.View;
import android.view.ViewGroup;

import com.fastaccess.R;
import com.fastaccess.data.dao.model.User;
import com.fastaccess.provider.scheme.LinkParserHelper;
import com.fastaccess.ui.widgets.AvatarLayout;
import com.fastaccess.ui.widgets.FontTextView;
import com.fastaccess.ui.base.adapter.BaseRecyclerAdapter;
import com.fastaccess.ui.base.adapter.BaseViewHolder;


/**
 * Created by Kosh on 11 Nov 2016, 2:08 PM
 */

public class UsersViewHolder extends BaseViewHolder<User> {

    AvatarLayout avatar;
    FontTextView title;
    FontTextView date;
    private final boolean isFilter;

    private UsersViewHolder(@NonNull View itemView, @Nullable BaseRecyclerAdapter<User, UsersViewHolder, BaseViewHolder.OnItemClickListener<User>> adapter, boolean isFilter) {
        super(itemView, adapter);
        this.isFilter = isFilter;
        this.avatar = itemView.findViewById(R.id.avatarLayout);
        this.title = itemView.findViewById(R.id.title);
        this.date = itemView.findViewById(R.id.date);
    }

    public static UsersViewHolder newInstance(@NonNull ViewGroup parent, @Nullable BaseRecyclerAdapter<User, UsersViewHolder, BaseViewHolder.OnItemClickListener<User>> adapter, boolean isFilter) {
        return new UsersViewHolder(getView(parent, isFilter ? R.layout.users_small_row_item : R.layout.feeds_row_item), adapter, isFilter);
    }

    @Override public void onClick(View v) {
        if (isFilter) {
            super.onClick(v);
        } else {
            avatar.findViewById(R.id.avatar).callOnClick();
        }
    }

    @Override public void bind(@NonNull User user) {}

    public void bind(@NonNull User user, boolean isContributor) {
        avatar.setUrl(user.getAvatarUrl(), user.getLogin(), user.isOrganizationType(),
                LinkParserHelper.isEnterprise(user.getHtmlUrl()));
        title.setText(user.getLogin());
        date.setVisibility(!isContributor ? View.GONE : View.VISIBLE);
        if (isContributor) {
            date.setText(String.format("%s (%s)", date.getResources().getString(R.string.commits), user.getContributions()));
        }
    }
}
